/*************************************************************************
 * AmarokNowPlaying2 - NowPlaying script for Amarok 2.                   *
 * See NowPlaying <http://nowplaying.heliohost.org/>                     *
 * Copyright (C) 2009-2011 Jany Belluz                                   *
 *                                                                       *
 * This program is free software: you can redistribute it and/or modify  *
 * it under the terms of the GNU General Public License as published by  *
 * the Free Software Foundation, either version 3 of the License, or     *
 * (at your option) any later version.                                   *
 *                                                                       *
 * This program is distributed in the hope that it will be useful,       *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 * GNU General Public License for more details.                          *
 *                                                                       *
 * You should have received a copy of the GNU General Public License     *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 *************************************************************************/

var NowPlayingDefaults = {};

// Version
NowPlayingDefaults.Version = "1.1";
NowPlayingDefaults.Id = "nowplaying_for_amarok_2";

// Default configuration, you can modify this :
// Configuration par défaut, vous pouvez bidouiller :

// Where the AmarokNowPlaying PHP scripts are hosted:
// Without trailing "/", without "/api" at the end
// Adresse à laquelle sont hébergés les scripts PHP AmarokNowPlaying :
// Sans "/" ni "/api" à la fin
//NowPlayingDefaults.RemoteAppBase = "http://nowplaying.host-ed.net";
NowPlayingDefaults.RemoteAppBase = "http://nowplaying.heliohost.org";

// Remote scripts version number (former AmarokNowPlaying is 0,
// new NowPlaying is 1)
// Numéro de version des scripts distants (l'ancien AmarokNowplaying est la version 0,
// le nouveau NowPlaying est la 1)
NowPlayingDefaults.RemoteAppVersion = 1;

// Default size of album cover thumbnails, in pixels:
// Taille par défaut des miniatures de pochette d'album, en pixels:
NowPlayingDefaults.CoverSize = 75;

// Proxy
NowPlayingDefaults.Proxy = {
	"enabled" : false,
	"host" : "example.com",
	"port" : "8080",
	"user" : "johndoe",
	"password" : "pouet"
};

// Debug mode:
NowPlayingDefaults.ShowDebug = false;

// Beta function:
NowPlayingDefaults.EnableBetaFeatures = false;

// Dependencies
Importer.loadQtBinding( "qt.core" );
Importer.loadQtBinding( "qt.gui" );
Importer.loadQtBinding( "qt.network" );
Importer.loadQtBinding( "qt.uitools" );


/* POST network requests handling class
 */
var DataPOSTer = {
	/* Posts data to url
	 * @param data (object) associative object of {key1: "value1", key2: "value2"...}
	 * @param HTTPParams (object) associative object of {key1: "value1", key2: "value2"...}
	 * @return (QHttp*) modified QHttp object
	 */
	simple: function(url, data, HTTPParams) {
		// Do normal POST
		var postRequest = new QHttpRequestHeader();
		if ( HTTPParams ) {
			for ( var i in HTTPParams ) {
				postRequest.setValue(i, HTTPParams[i]);
			}
		}
		
		postRequest.setRequest("POST", url.encodedPath().toString(), 1, 1);
		postRequest.setValue("Host", url.encodedHost().toString());
		postRequest.setContentType("application/x-www-form-urlencoded");
		
		var postDataByteArray = new QByteArray();
		var first = true;
		for ( var i in data ) {
			if ( !first ) {
				postDataByteArray.append("&");
			} else {
				first = false;
			}
			postDataByteArray.append(i+"="+QUrl.toPercentEncoding(data[i]));
		}
		
		postRequest.setContentLength(postDataByteArray.length());
		
		return DataPOSTer.getCustomQHttp(url, postDataByteArray, postRequest);
	},
	
	/* Posts data to url, including files, using multipart
	 * @param data (object) associative object of {key1: "value1", ..., cover: QIODevice}
	 * @param HTTPParams (object) associative object of {key1: "value1", key2: "value2"...}
	 * @return (QHttp*) modified QHttp object
	 */
// 	multipart: function(url, data, HTTPParams) {
// 		var postRequest = new QHttpRequestHeader();
// 		if ( HTTPParams ) {
// 			for ( var i in HTTPParams ) {
// 				postRequest.setValue(i, HTTPParams[i]);
// 			}
// 		}
// 		
// 		var boundary = "paranguaricutirimirruaru0xdeadbeefpouet";
// 		postRequest.setRequest("POST", url.encodedPath().toString(), 1, 1);
// 		postRequest.setValue("Host", url.encodedHost().toString());
// 		postRequest.setContentType("multipart/form-data; boundary=\""+boundary+"\"");
// 		
// 		var postDataByteArray = new QByteArray();
// 		
// 		for ( var i in data ) {
// 			if ( i != "cover" ) {
// 				postDataByteArray.append("--"+boundary+"\r\n");
// 				postDataByteArray.append("content-disposition: form-data; name=\""+i+"\"\r\n");
// 				postDataByteArray.append("\r\n");
// 				postDataByteArray.append(data[i]+"\r\n");
// 			}
// 		}
// 		
// 		// Deal with files, data.cover is a QBuffer.
// 		data.cover.open(QIODevice.ReadOnly);
// 		postDataByteArray.append("--"+boundary+"\r\n");
// 		postDataByteArray.append("content-disposition: form-data; name=\"cover\"; filename=\""+data.cover.fileName+"\"\r\n");
// 		postDataByteArray.append("Content-Type: image/png\r\n");
// 		postDataByteArray.append("Content-Transfer-Encoding: binary\r\n");
// 		postDataByteArray.append("\r\n");
// 		postDataByteArray.append(data.cover.readAll());
// 		postDataByteArray.append("\r\n--"+boundary+"--\r\n");
// 		data.cover.close();
// 		
// 		postRequest.setContentLength(postDataByteArray.length());
// 		
// 		return DataPOSTer.getCustomQHttp(url, postDataByteArray, postRequest);
// 	},
	
	/* Makes a custom QHttp object
	 * Don't call it yourself!
	 * @return (QHttp*) modified QHttp object
	 * @sa simple(), multipart()
	 */
	getCustomQHttp: function(url, byteArrayData, requestHeader) {
		var http = new QHttp();
		debugMessage("host: "+url.encodedHost(), "DataPOSTer");
		if (NowPlaying.Proxy.enabled) {
			var proxy = new QNetworkProxy();
			proxy.setType(QNetworkProxy.HttpProxy);
			proxy.setHostName(NowPlaying.Proxy.host);
			proxy.setPort(NowPlaying.Proxy.port);
			proxy.setUser(NowPlaying.Proxy.user);
			proxy.setPassword(NowPlaying.Proxy.password);
			http.setProxy(proxy);
		}
		http.setHost(url.encodedHost());
		http.requestHeader = requestHeader;
		http.data = byteArrayData;
		return http;
	}
};

/* Sends informations to remote server.
 */
function sendData( data ) { try {
	var url = new QUrl(NowPlaying.RemoteAppBase +
		(NowPlaying.RemoteAppVersion == 1 ? "/api" : "") + "/post.php");
	var HTTPParams = {"User-Agent": "NowPlaying script"}; // TODO define what to put here
	
	data.pseudo = NowPlaying.UserPseudo;
	data.password = NowPlaying.UserPassword;
	
	var http = DataPOSTer.simple(url, data, HTTPParams);
	
	http.param = data.param;
	http.hadCover = (data.cover ? true : false);
	http.errorFlag = false;
	http.done.connect( http, "nowPlayingProcessReply" );
	http.go();
} catch (err) { debugException(err); } }

/* Gets status codes and such from a network request
 */
QHttp.prototype.nowPlayingProcessReply = function (error) { try {
	if ( !error ) {
		var replyData = this.readAll();
		
		debugMessage(replyData.toString(), "HTTP");
		
		if ( this.param == "update" )
			timer.start(1000 * 60 * 10); // 10 minutes
		
		if ( replyData.indexOf("OK") != -1 ) {
			if ( this.param == "none" || this.param == "post" ) {
				if ( this.hadCover && replyData.indexOf("COVER") != -1 ) {
					Amarok.Window.Statusbar.shortMessage(qsTr("NowPlaying - Track information sent, with cover.") );
				} else {
					Amarok.Window.Statusbar.shortMessage(qsTr("NowPlaying - Track information sent.") );
				}
			}
			else if ( this.param == "special" ) {
				if ( this.hadCover && replyData.indexOf("COVER") != -1 ) {
					Amarok.Window.Statusbar.shortMessage(qsTr("NowPlaying - Special message sent, with cover.") );
				} else {
					Amarok.Window.Statusbar.shortMessage(qsTr("NowPlaying - Special message sent.") );
				}
			}
			else if ( this.param == "update" ) {
				timer.stop();
				if ( replyData.indexOf("NEWVERSION") != -1 ) {
					new_version = replyData.match(/~([0-9]+\.)+[0-9]+~/)[0].replace('~', '');
					website_url = replyData.match(/~https?:\/\/.+~/)[0].replace('~', '');
					showMessage(qsTr("A newer version of this script is available, check its website: %1").
					replace("%1", website_url), tr('Update'));
				}
			}
			else { // this.param = "test"
				Amarok.Window.Statusbar.shortMessage(qsTr("NowPlaying - Your pseudo and password are ok.") );
			}
		}
		else if ( replyData.indexOf("QUERY") != -1 ) {
			showMessage(qsTr("NowPlaying - Error while posting to server:")+"\n"+
				qsTr("Request was malformed. You should consider upgrading your script."), "Warning");
		}
		else if ( replyData.indexOf("AUTH") != -1 ) {
			showMessage(qsTr("NowPlaying - Authentication error:")+"<br />"+
				(qsTr("Your pseudo or your password is wrong, you should configure NowPlaying from the Settings menu, or visit <a href=\"%1\">NowPlaying's website</a>, so as to sign up and choose a pseudo and a password.").
				replace("%1", NowPlaying.RemoteAppBase+"/"))+"\n"+
				qsTr("NowPlaying has been disabled."), "Warning", "RichText");
			Amarok.Script.writeConfig("useNowPlaying", "false");
			NowPlaying.UseNowPlaying = false;
		}
		else {
			Amarok.Window.Statusbar.longMessage(qsTr("NowPlaying - Unknown error:")+
				"\n"+qsTr("An unknown error occured.") +
				((new Number(this.lastResponse().statusCode() / 100).toFixed(0)) == 3 ?
				"\n"+qsTr("We got a redirection while trying to post to remote server.") : "" ));
		}
	}
	else {
		Amarok.Window.Statusbar.longMessage(qsTr("NowPlaying - Network Error:")+
		"\n"+this.errorString());
	}
	lastProcessedEventId = 0;
} catch (err) { debugException(err, "function QHttp.prototype.nowPlayingProcessReply"); } };

QHttp.prototype.go = function() { try {
	if ( this.requestHeader && this.data ) {
		debugMessage(this.requestHeader.toString(), "QHttp.go()");
		this.request(this.requestHeader, this.data);
	}
} catch (err) { debugException(err, "function QHttp.prototype.go"); } };

/* Checks if a new version of this script is available
 */
function checkUpdate() { try {
	if ( Amarok.Script.readConfig("useNowPlaying", "true") == "true" ) {
		var data = {};
		data.param = "update";
		data.player = "Amarok";
		data.player_version = Amarok.Info.version();
		data.script = NowPlaying.Id;
		data.script_version = NowPlaying.Version;
		sendData(data);
	}
} catch (err) { debugException(err, "function checkUpdate"); } }

/* Gets from Amarok all needed information, then gets cover image, and puts
 * everything in an object.
 */
function fetchTrackData() { try {
	if (Amarok.Engine.engineState() == 2) {
		// Amarok is on "stop"
		return false;
	}
	
	var data = {};
	
	data.title = Amarok.Engine.currentTrack().title;
	data.artist = Amarok.Engine.currentTrack().artist;
	data.album = Amarok.Engine.currentTrack().album;
	data.score = Amarok.Engine.currentTrack().score;
	data.rating = Amarok.Engine.currentTrack().rating;
	data.counter = Amarok.Engine.currentTrack().playCount;
	data.genre = Amarok.Engine.currentTrack().genre;
	data.length = (Amarok.Engine.currentTrack().length >= 3600000 ?
		(new QTime()).addMSecs(Amarok.Engine.currentTrack().length - 0).toString("hh:mm:ss") :
		(new QTime()).addMSecs(Amarok.Engine.currentTrack().length - 0).toString("mm:ss"));
	data.year = Amarok.Engine.currentTrack().year;
	switch ( NowPlaying.RemoteAppVersion ) {
		case 0:
			data.amarok_ver = Amarok.Info.version();
			break;
		case 1:
		default:
			data.player = "Amarok";
			data.player_version = Amarok.Info.version();
			break;
	}
	
	// Cover ?
	if ( NowPlaying.SendAlbumCover ) { /*  && 
		(Amarok.Info.version() + "").indexOf("2.0") == -1 Check for Amarok >= 2.1 */
		try {
			if ( Amarok.Engine.currentTrack().imageUrl != '' ) {
				var pixmap = Amarok.Engine.currentTrack().imagePixmap(NowPlaying.CoverSize);
				var buffer = new QBuffer();
				buffer.open(QIODevice.WriteOnly);
				pixmap.save(buffer, "PNG"); // writes pixmap into bytes in PNG format
				buffer.close();
				buffer.open(QIODevice.ReadOnly);
				data.cover = buffer.readAll().toBase64().toString();
				buffer.close();
				data.cover_post_base64_method = "true";
			}
			else {
				data.cover = false;
			}
		}
		catch (e) {
			data.cover = false;
		} // When no track playing, no pixmap -> error!
	}
	
	return data;
} catch (err) { debugException(err, "function fetchTrackData"); } }

/* Called on 'special' post event
 */
function specialPostEventHandler() { try {
	specialPostDataCache = fetchTrackData();
	if (specialPostDataCache !== false) {
		specialPostDataCache.param = "special";
		
		// Add info to SongInfo label
		specialPostDialog.frame_SongInfo.label_SongInfo.setText(
			"<strong>%TITLE%</strong> by <strong>%ARTIST%</strong> on <em>%ALBUM%</em>"
			.replace(/%TITLE%/g, specialPostDataCache.title)
			.replace(/%ARTIST%/g, specialPostDataCache.artist)
			.replace(/%ALBUM%/g, specialPostDataCache.album)
		);
	}
	else {
		specialPostDialog.frame_SongInfo.label_SongInfo.setText(
			qsTr("No song to comment on!"));
	}
	
	specialPostDialog.lineEdit_Message.setEnabled(specialPostDataCache !== false);
	specialPostDialog.textEdit_Comment.setEnabled(specialPostDataCache !== false);
	specialPostDialog.label_Message.setEnabled(specialPostDataCache !== false);
	specialPostDialog.label_Comment.setEnabled(specialPostDataCache !== false);
	
	// Reset fields in specialPostDialog
	specialPostDialog.lineEdit_Message.setText("");
	specialPostDialog.textEdit_Comment.setText("");
	
	// TODO reset previews
	
	specialPostDialog.show();
} catch (err) { debugException(err, "function specialPostEventHandler"); } }

/* Called when specialPostDialog is accepted
 */
function processSpecialPost() { try {
	if (specialPostDataCache !== false) {
		specialPostDataCache.special_message = specialPostDialog.lineEdit_Message.text;
		specialPostDataCache.special_comment = specialPostDialog.textEdit_Comment.plainText;
		// TODO fetch data from dialog about wich handlers to use
		
		sendData(specialPostDataCache);
	}
} catch (err) { debugException(err, "function processSpecialPost"); } }

/* Puts configuration in the NowPlaying object.
 */
function readConfig() { try {
	with (Amarok.Script) {
		NowPlaying.UseNowPlaying = ( readConfig("useNowPlaying", "true") == "true" );
		NowPlaying.SendAlbumCover = ( readConfig("sendAlbumCover", "true") == "true" );
		NowPlaying.CoverSize = readConfig("coverSize", NowPlayingDefaults.CoverSize + "") - 0;
		
		NowPlaying.UserPseudo = readConfig("userPseudo", "") + "";
		NowPlaying.UserPassword = readConfig("userPassword", "") + "";
		
		NowPlaying.RemoteAppBase = readConfig("remoteAppBase", NowPlayingDefaults.RemoteAppBase + "") + "";
		NowPlaying.RemoteAppVersion = readConfig("remoteAppVersion", NowPlayingDefaults.RemoteAppVersion + "") - 0;
		NowPlaying.ShowDebug = ( readConfig("showDebug", NowPlayingDefaults.ShowDebug + "") == "true" );
		NowPlaying.EnableBetaFeatures = ( readConfig("enableBetaFeatures", NowPlayingDefaults.EnableBetaFeatures + "") == "true" );
		
		NowPlaying.Proxy = {};
		NowPlaying.Proxy.enabled =
			(readConfig("proxyEnabled", "" + NowPlayingDefaults.Proxy.enabled) == "true");
		NowPlaying.Proxy.host =
			"" + readConfig("proxyHost", "" + NowPlayingDefaults.Proxy.host);
		NowPlaying.Proxy.port =
			readConfig("proxyPort", "" + NowPlayingDefaults.Proxy.port) - 0;
		NowPlaying.Proxy.user =
			"" + readConfig("proxyUser", "" + NowPlayingDefaults.Proxy.user);
		NowPlaying.Proxy.password =
			"" + readConfig("proxyPassword", "" + NowPlayingDefaults.Proxy.password);
	}
		
} catch (err) { debugException(err, "function configureEventHandler"); } }

/* Called on configure event
 */
function configureEventHandler() { try {
	with (configureDialog.tabWidget.children()[0]) {
		// General
		tab_General.checkBox_UseNowPlaying.setChecked(NowPlaying.UseNowPlaying);
		tab_General.checkBox_SendAlbumCover.setChecked(NowPlaying.SendAlbumCover);
		tab_General.spinBox_CoverSize.setValue(NowPlaying.CoverSize);
		
		// Login
		tab_Login.lineEdit_Pseudo.setText(NowPlaying.UserPseudo);
		tab_Login.lineEdit_Password.setText(
			(NowPlaying.UserPassword != "" ? "dummy!" : "") );
		
		// Advanced
		tab_Advanced.lineEdit_RemoteAppBase.setText(NowPlaying.RemoteAppBase);
		tab_Advanced.comboBox_RemoteAppVersion.setCurrentIndex(NowPlaying.RemoteAppVersion);
		tab_Advanced.checkBox_ShowDebug.setChecked(NowPlaying.ShowDebug);
		tab_Advanced.checkBox_EnableBetaFeatures.setChecked(NowPlaying.EnableBetaFeatures);
		
		// Proxy
		tab_Proxy.checkBox_ProxyEnabled.setChecked(NowPlaying.Proxy.enabled);
		tab_Proxy.lineEdit_ProxyHost.setText(NowPlaying.Proxy.host);
		tab_Proxy.spinBox_ProxyPort.setValue(NowPlaying.Proxy.port);
		tab_Proxy.lineEdit_ProxyUser.setText(NowPlaying.Proxy.user);
		tab_Proxy.lineEdit_ProxyPassword.setText(NowPlaying.Proxy.password);
	}
	
	configureDialog.show(); 
} catch (err) { debugException(err, "function configureEventHandler"); } }

/* Called when configure dialog is accepted
 */
function saveConfiguration() { try {
	with (configureDialog.tabWidget.children()[0]) {
		Amarok.Script.writeConfig("useNowPlaying",
			tab_General.checkBox_UseNowPlaying.checked + "");
		Amarok.Script.writeConfig("sendAlbumCover",
			tab_General.checkBox_SendAlbumCover.checked + "");
		Amarok.Script.writeConfig("coverSize",
			tab_General.spinBox_CoverSize.value + "");
		
		Amarok.Script.writeConfig("userPseudo",
			tab_Login.lineEdit_Pseudo.text + "");
		if ( tab_Login.lineEdit_Password.modified ) {
			var userPassword = tab_Login.lineEdit_Password.text;
			Amarok.Script.writeConfig("userPassword",
				QCryptographicHash.hash(QUrl.toPercentEncoding(userPassword+"_Pouet!", "", "~"), QCryptographicHash.Md5).toHex() + "");
		}
		
		Amarok.Script.writeConfig("remoteAppBase",
			tab_Advanced.lineEdit_RemoteAppBase.text + "");
		Amarok.Script.writeConfig("remoteAppVersion",
			tab_Advanced.comboBox_RemoteAppVersion.currentIndex + "");
		Amarok.Script.writeConfig("showDebug",
			tab_Advanced.checkBox_ShowDebug.checked + "");
		Amarok.Script.writeConfig("enableBetaFeatures",
			tab_Advanced.checkBox_EnableBetaFeatures.checked + "");
		
		Amarok.Script.writeConfig("proxyEnabled",
			tab_Proxy.checkBox_ProxyEnabled.checked + "");
		Amarok.Script.writeConfig("proxyHost",
			tab_Proxy.lineEdit_ProxyHost.text + "");
		Amarok.Script.writeConfig("proxyPort",
			tab_Proxy.spinBox_ProxyPort.value + "");
		Amarok.Script.writeConfig("proxyUser",
			tab_Proxy.lineEdit_ProxyUser.text + "");
		Amarok.Script.writeConfig("proxyPassword",
			tab_Proxy.lineEdit_ProxyPassword.text + "");
	}
	
	readConfig();
	initDebug();
	initBeta();
	sendData({param: "test"});
} catch (err) { debugException(err, "function saveConfiguration"); } }

/* Fill the proxy config tab with data from http_proxy env variable
 */
function proxyGuessFromEnvEvent() { try {
	var env = QProcess.systemEnvironment();
	var l = env.length;
	var url = "";
	for (var i = 0; i < l; ++i) {
		if (matches = env[i].match(/^http_proxy\s?=\s?(.+)/i)) {
			url = matches[1];
		}
	}
	if (url != "") {
		qUrl = new QUrl(url);
		with (configureDialog.tabWidget.children()[0]) {
			tab_Proxy.checkBox_ProxyEnabled.setChecked(true);
			tab_Proxy.lineEdit_ProxyHost.setText(qUrl.host());
			tab_Proxy.spinBox_ProxyPort.setValue(qUrl.port());
			tab_Proxy.lineEdit_ProxyUser.setText(qUrl.userName());
			tab_Proxy.lineEdit_ProxyPassword.setText(qUrl.password());
		}
	}
	
} catch (err) { debugException(err, "function proxyGuessFromEnvEvent"); } }

function configDiagRestoreDefaults() { try {
	configureDialog.tabWidget.children()[0].tab_Advanced.lineEdit_RemoteAppBase.setText(NowPlayingDefaults.RemoteAppBase);
	configureDialog.tabWidget.children()[0].tab_Advanced.comboBox_RemoteAppVersion.setCurrentIndex(NowPlayingDefaults.RemoteAppVersion);
} catch (err) { debugException(err, "function configDiagRestoreDefaults"); } }

/* Called on track event (play/pause/stop)
 */
function trackEventHandler() { try {
	if (NowPlaying.UseNowPlaying) {
		// Make an id for this request (workaround)
		var eventId;
		try {
			eventId = "pouet" + Amarok.Engine.engineState() + Amarok.Engine.currentTrack().title + Amarok.Engine.currentTrack().artist + Amarok.Engine.currentTrack().album;
		}
		catch(e) {
			eventId = "pouet" + Amarok.Engine.engineState();
		}
		
		if ( eventId == lastProcessedEventId ) {
			return;
		}
		lastProcessedEventId = eventId;
		
		// Determine wich param to use: none or post
		var postData = {};
		var state = Amarok.Engine.engineState(); // 0: playing, 1: pause, 2: stop
		switch ( state ) {
			case 0:
				// Add track informations
				postData = fetchTrackData();
				postData.param = "post";
				break;
			case 1:
			case 2:
				postData.param = "none";
				break;
		}
		sendData(postData);
	}
} catch (err) { debugException(err, "function trackEventHandler"); } }

/* Refresh preview of the special post
 */
function specialPostRefreshPreview() { try {
	// TODO
} catch (err) { debugException(err, "function specialPostRefreshPreview"); } }

/* Shows message as Amarok.Alert()
 */
function showMessage(message, icon, format) { try {
	if ( icon == "" || icon == undefined ) {
		icon = "Information";
	}
	var diag = new QMessageBox(QMessageBox[icon], "NowPlaying - Amarok", message);
	if ( format ) {
		diag.textFormat = Qt[format];
	}
	diag.setParent(this);
	diag.show();
} catch (err) { debugException(err, "function showMessage"); } }

/* Debug function which works with exceptions.
 */
function debugException(e, context) {
	var data = "NowPlaying - Error:\n\nMessage: "+e+"\n"+"Context: "+context+"\n\n"+
		"An exception was found. You should upgrade your script, or, "+
		"if you have the latest version, submit a bug report. Thanks!";
// 		for ( var i in e ) {
// 			data += i+": "+e[i]+"\n";
// 		}
	Amarok.alert(data);
}

/* Debug function
 */
function debugMessage(message, type) {
	if ( NowPlaying.ShowDebug ) {
		debug_text_box.append(
		"\n\n-----------------------------------------------------------\n"+
		type+" debug:"+"\n"+message
		);
	}
}

/* Debug function
 */
function debugObject(object, type) {
	if ( NowPlaying.ShowDebug ) {
		var data = "Object of type: "+typeof(object)+"\n";
		for ( var i in object ) {
			data += i+": "+object[i]+"\n";
		}
		debugMessage(data, type);
	}
}

function initDebug() { try {
	if ( NowPlaying.ShowDebug ) {
		if ( debug_text_box === 0 ) {
			debug_text_box = new QTextEdit();
			debug_text_box.setWindowTitle("NowPlaying debug");
			debug_text_box.readOnly = true;
			debug_text_box.setPlainText("NowPlaying debug");
			debug_text_box.resize(500, 400);
			debug_text_box.show();
		}
		else
			debug_text_box.show();
	}
	else if ( debug_text_box !== 0 ) {
		debug_text_box.hide();
	}
} catch (err) { debugException(err, "function initDebug"); } }

function initBeta() { try {
	if ( NowPlaying.EnableBetaFeatures ) {
		
	}
} catch (err) { debugException(err, "function initBeta"); } }

/* Workarounds
 */
QByteArray.prototype.toString = function() {
	ts = new QTextStream( this, QIODevice.ReadOnly );
	return ts.readAll();
};
function clone(o) {
  var newObj = (o instanceof Array) ? [] : {};
  for (i in o) {
      newObj[i] = o[i];
  }
  return newObj;
};

/* Main:
 * load i18n
 * load UI files
 * signal-slots connections
 */

try {

// Puts configuration in the NowPlaying object.
var NowPlaying = clone(NowPlayingDefaults);
readConfig();

// FIXME Load i18n
// var translator = new QTranslator();
// translator.load("NowPlayingTranslations", "/translations");
// QMainApplication.installTranslator(&translator);
function qsTr(p) {return String(p)}

// Load UI files
var UIloader = new QUiLoader( this );
var uiFile = new QFile ( Amarok.Info.scriptPath() + "/Configure.ui" );
uiFile.open( QIODevice.ReadOnly );
var configureDialog = UIloader.load( uiFile, this);
uiFile.close();

configureDialog.accepted.connect(saveConfiguration);
configureDialog.tabWidget.children()[0].tab_Advanced.pushButton_RestoreDefaults.clicked.connect(
	configDiagRestoreDefaults);
// example: configureDialog.tabWidget.children()[0].tab_Login.label_Pseudo
configureDialog.tabWidget.children()[0].tab_Login.label_NoticeLogin.setText(
	configureDialog.tabWidget.children()[0].tab_Login.label_NoticeLogin.text.replace(
	"%1", NowPlaying.RemoteAppBase+"/"));
configureDialog.tabWidget.children()[0].tab_Proxy.pushButton_ProxyGuess.clicked.connect(
	proxyGuessFromEnvEvent);

var specialPostDataCache;

uiFile = new QFile ( Amarok.Info.scriptPath() + "/SpecialPost.ui" );
uiFile.open( QIODevice.ReadOnly );
var specialPostDialog = UIloader.load( uiFile, this);
uiFile.close();

// specialPostDialog.lineEdit_Message.textEdited.connect(specialPostRefreshPreview);
// specialPostDialog.textEdit_Comment.textChanged.connect(specialPostRefreshPreview);
specialPostDialog.accepted.connect(processSpecialPost);


initBeta();

// Check new version each 10 minutes until done (see QHttp.prototype.nowPlayingProcessReply)
var timer = new QTimer();
timer.timeout.connect( checkUpdate );
timer.singleShot = true;
timer.start(1000 * 60 * 10); // 10 minutes

Amarok.Window.addSettingsSeparator();
Amarok.Window.addSettingsMenu( "npConfigure", qsTr("NowPlaying - Configure") );
Amarok.Window.SettingsMenu.npConfigure['triggered()'].connect( configureEventHandler );

Amarok.Window.addToolsSeparator();
Amarok.Window.addToolsMenu( "npSpecialPost", qsTr("NowPlaying - Comment this song") );
Amarok.Window.ToolsMenu.npSpecialPost['triggered()'].connect( specialPostEventHandler );

var lastProcessedEventId;
var eventDelay = new QTimer();
eventDelay.singleShot = true;
function delayTrackEventHandler() {
	eventDelay.start(2000);
}
eventDelay.timeout.connect( trackEventHandler );

Amarok.Engine.trackPlayPause.connect( delayTrackEventHandler );
Amarok.Engine.trackChanged.connect( delayTrackEventHandler );
Amarok.Engine.trackFinished.connect( delayTrackEventHandler );

var debug_text_box = 0;
initDebug();

debugObject(NowPlaying);
debugObject(NowPlayingDefaults);

} catch (err) { debugException(err, "main"); }
